/******************************************************************************
 *
 * $Source: /src/rcs/Cats/XSystem/pex/mbx/mbx_centerline.c,v $
 * $Date: 93/12/30 13:53:07 $
 * $Revision: 1.3 $
 *
 * Description:  
 *	This test is for:						 
 *		1. Mainline MBX buffering with Renderer clearing image
 *		2. Destroy buffers and recreation of them
 *		3. Mainline MBX buffering with server clearing image to
 *		   background.
 *
 * Written by: Ken Tidwell
 * Ported to susPEX by Norm Gee
 *
 *****************************************************************************/


/******************************************************************************
 *
 * Include Files  
 *               
 ******************************************************************************/

/* ------ system and platform files ------------------------------------------*/
#include <X11/PEX5/PEXlib.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/extensions/multibuf.h>
#include <stdio.h>
#include <math.h>

/* ------ program specific ---------------------------------------------------*/

#include <lut.h>
#include <misc.h>
#include "pexutcmap.h"
#include "common.h"

#include "common.c"

void testproc(Display *dpy, Window window, int cur_image);

void inquire_test_params(char *test_name, 
			 int  *num_images, 
			 int  *supported_modes, 
			 char *win_title)
{
    strcpy(test_name, "mbx_centerline");
    *num_images      = 5;
    *supported_modes = IMM_MODE;

    strcpy(win_title, "MBX Centerline");
}


/******************************************************************************/
void
main( argc, argv )
	int  argc;
	char **argv;
/******************************************************************************/
{
    Display		       *Dpy;
    Window			Win;
    Drawable			drawable_to_use;
    int                         Screen=0;
    XEvent			Event;
    XColor			BackGroundColor;
    XStandardColormap		ColorMapInfo;
    XVisualInfo			VisualInfo;
    PEXUtVisualCriteria         Criteria;
    unsigned int                UnmetCriteria;
    PEXUtWindowSpecification    WindowInfo;
    Atom                        StdPropAtom;
    PEXColorApproxEntry		CapxInfo;
    PEXRenderer			Renderer;
    PEXRendererAttributes	RAttrs;
    PEXExtensionInfo	       *pexinfo;
    int				Done = 0,
				CubeType = WIREFRAME, /* Default WIREFRAME    */
                                ColorFlag = REDLINES; /* Default RED Cube     */
    double                      DeltaVal=WIREFRAMEDELTA, /* Rotation increment*/
				Angle=0;              /* Rotation angle       */
    XSizeHints 			Hints;
    int 			Result;
    char			err[80];
    /*
    ** Variables for tracking MBX operations. We do not support more than
    ** 2 buffers currently. I am writing this test framework to be extensible
    ** to multiple buffers in the future. -KT
    */
    Multibuffer			buffers[4], displayed_buffer, render_buffer;
    int				num_buffers;
    
    /*
    ** Variables for finding out about MBX version.
    */
    int 			mbx_event_base, mbx_error_base;
    int 			server_major_version, server_minor_version;
    Status			status;

    /*
    ** Other Variables for susPEX
    */
    int			pexmode = 0;
    int image_depth = 0;


    /*
    ** Open a display and initialize PEX.
    */
    
    Dpy = init_pex( argv, &pexinfo );
    if ( Dpy == (Display *)NULL )
    {
        exit(1);
    }

    /*
     * Set up error handler
     */
    _hppex_test_init_error_handler();

    /*  Handle window geometry string if present */
    parse_window_geometry(argc, argv);

    /* Handle other arguments. */
    if (parse_options (argc, argv, &pexmode, &image_depth))
	exit (1);


    /*
    ** Set up properties of the window
    */
    Screen = DefaultScreen(Dpy);
    Hints.x = XPOS;
    Hints.y = YPOS;
    Hints.width = WIDTH;
    Hints.height = HEIGHT;
    Hints.flags = (USSize|USPosition);

    /*
    ** Vary PEXUtDoubleBufferingCapability to get single 24 bit buffer on
    ** rattler.
    */
    Criteria.hard_criteria_mask = PEXUtMinColors | 
				  PEXUtDoubleBufferingCapability;
    Criteria.soft_criteria_mask = PEXUtStandardColormapProperty;
    Criteria.min_colors = 2;
    Criteria.standard_colormap_property = True;
    Criteria.double_buffering_capability = PEXUtDbufferPEX;

    WindowInfo.attr_mask = 0;
    WindowInfo.title = "Double-Buffering";
    WindowInfo.size_hints = Hints;
    WindowInfo.parent = RootWindow (Dpy, Screen);
    WindowInfo.border_width = 0;
    WindowInfo.background_color_name = "black";
    WindowInfo.border_color_name = "blue";

    /*
    ** Create window and colormap
    */
    Result = PEXUtCreateWindowAndColormap (Dpy, Screen, &Criteria,
					    &WindowInfo, &Win,
					    &VisualInfo, &ColorMapInfo,
					    &CapxInfo,
					    &UnmetCriteria, &StdPropAtom,
					    &BackGroundColor);

    if ( Win == None )
    {
	fprintf (stderr, "Could not create window and colormap!\n");
	exit (2);
    }

    XSetWindowColormap(Dpy, Win, ColorMapInfo.colormap);
    XInstallColormap(Dpy, ColorMapInfo.colormap);
    XSync(Dpy, 0);

    /*
    ** Create renderer.
    */
    Renderer = My_setup_renderer( Dpy, Win, &CapxInfo, &RAttrs );

    /*
    ** Set the renderer to clear image at BeginRendering.
    ** Turn Z buffer on.
    */
    RAttrs.clear_image = True;
    RAttrs.hlhsr_mode = PEXHLHSRZBuffer;
    PEXChangeRenderer (Dpy, Renderer, PEXRAHLHSRMode | PEXRAClearImage,
		       &RAttrs);


    /**************************************************************************
    ** Setup MBX
    ***************************************************************************
    */

    /*
    ** STEP 1
    **
    ** Setup and configure MBX buffering.
    */

    /* 
    ** 1A
    ** 
    ** See if MBX is there.
    */
    if (! XmbufQueryExtension (Dpy, &mbx_event_base, &mbx_error_base))
    {
	fprintf(stderr,"MBX extension not found!\n");
	exit (3);
    }
   
    /*
    ** 1B
    **
    ** See what version of MBX is there. We want to use 3.2 protocol, which 
    ** corresponds to server version 1.0, which is what we check for.
    */
    status = XmbufGetVersion (Dpy, &server_major_version, 
			      &server_minor_version);
    if ((! status) || (server_major_version != 1) || 
	(server_minor_version != 0))
    {
	fprintf(stderr,"MBX version not compatible!\n");
	exit (4);
    }

    /*
    ** 1C
    **
    ** Create  buffers.
    **
    */
    num_buffers =  XmbufCreateBuffers (Dpy, Win, 2, 
				       MultibufferUpdateActionUndefined,
				       MultibufferUpdateHintFrequent,
				       buffers);
    if (num_buffers != 2)
    {
	fprintf(stderr,"Number of buffers requested not allocated!\n");
	exit (5);
    }

    /*
    ** Displayed buffer is first buffer in buffers list returned. Render
    ** buffer will be determined in render portion of test.
    */
    displayed_buffer = 0;

   
    /*
    ** Ensure the output buffer is flushed (without discarding events).
    */
    XSync ( Dpy, False );

    /*
    **
    ** Create the cube and set the correct view
    */
    SetColors ( Dpy, RAttrs.color_table);
    SetView( Dpy, RAttrs.view_table );

    /*************************************************************************
    **
    ** TEST CASES BEGIN HERE
    **
    *************************************************************************/

    /*
    ** 
    ** TEST 1  Double Buffer with Update Action Undefined
    **
    */
    describe_test("TEST 1: Simple double-buffering, update action Undefined.\n\n");


    /*
    ** Redraw the cube RED and solid.
    */
    render_buffer = 1;

    redraw( Dpy, buffers[render_buffer], Renderer, (Angle*M_PI)/180.0, 
	    REDLINES, SOLID);

    /*
    ** Check results 1. 
    */
    describe_test("Frame 1: Window should be blank. Red cube is in back buffer\n");
    testproc(Dpy, Win, 1);
    
    /*
    ** Swap buffers
    */
    XmbufDisplayBuffers (Dpy, 1, &buffers[render_buffer], 0, 0);
    XSync ( Dpy, False );  /* Push request over wire so we can check image. */
    displayed_buffer = render_buffer;
    render_buffer = !render_buffer;

    /*
    ** Check results 2. 
    */
    describe_test ("Frame 2: Buffers swapped. Red cube should be visible\n");

    testproc(Dpy, Win, 2);
    

    /*
    ** Redraw the cube Green and wireframe at an angle.
    */

    Angle += DeltaVal;
    redraw( Dpy, buffers[render_buffer], Renderer, (Angle*M_PI)/180.0, 
	    GREENLINES, WIREFRAME);

    /*
    ** Check results 3. 
    */
    describe_test ("Frame 3: Red cube should be visible. Green wire cube in back.\n");

    testproc(Dpy, Win, 3);
    

    /*
    ** Swap buffers
    */
    XmbufDisplayBuffers (Dpy, 1, &buffers[render_buffer], 0, 0);
    XSync ( Dpy, False );  /* Push request over wire so we can check image. */
    displayed_buffer = render_buffer;
    render_buffer = !render_buffer;

    /*
    ** Check results 4. 
    */
    describe_test ("Frame 4: Buffers swapped. Green wire cube should be visible.\n");

    testproc(Dpy, Win, 4);



    /*
    ** 
    ** TEST 2  Destroy buffers
    **
    */
    describe_test("TEST 2: Destroy buffers and recreate\n\n");

    /*
    ** Swap buffers
    */
    XmbufDestroyBuffers (Dpy, Win);
    XSync ( Dpy, False );  /* Push request over wire so we can check image. */

    /*
    ** Check results 1. 
    */
    describe_test ("Frame 5: Buffers Destroyed. Green wire should still be visible.\n");

    testproc(Dpy, Win, 5);


    /*
    ** Change renderer to not clear.
    */
    RAttrs.clear_image = False;
    PEXChangeRenderer (Dpy, Renderer, PEXRAClearImage, &RAttrs);

    /*
    ** Get new set of buffers.
    */
    num_buffers =  XmbufCreateBuffers (Dpy, Win, 2, 
				       MultibufferUpdateActionBackground,
				       MultibufferUpdateHintFrequent,
				       buffers);
    if (num_buffers != 2)
    {
	fprintf(stderr,"Number of buffers requested not allocated!\n");
	exit (6);
    }

    /*
    ** Displayed buffer is first buffer in buffers list returned. Render
    ** buffer will be determined in render portion of test.
    */
    displayed_buffer = 0;

    /*
    ** Select render_buffer to be back
    */
    render_buffer = 1; /* Back buffer */




    /*
    ** 
    ** TEST 3  Double Buffer with Update Action Background.
    **
    */
    describe_test("TEST 3: Simple double-buffering, update action Background.\n\n");

    /*
    ** Redraw the cube RED and solid.
    */
    redraw( Dpy, buffers[render_buffer], Renderer, (Angle*M_PI)/180.0, 
	    REDLINES, SOLID);

    /*
    ** Check results 1. 
    */
    describe_test ("Frame 6: Green wire cubeshould still be visible. \n");

    testproc(Dpy, Win, 6);
    
    /*
    ** Swap buffers
    */
    XmbufDisplayBuffers (Dpy, 1, &buffers[render_buffer], 0, 0);
    XSync ( Dpy, False );  /* Push request over wire so we can check image. */
    displayed_buffer = render_buffer;
    render_buffer = !render_buffer;

    /*
    ** Check results 2. 
    */
    describe_test ("Frame 7: Buffers swapped. Red cube should be visible\n");
    
    testproc(Dpy, Win, 7);

    /*
    ** Redraw the cube Green and wireframe at a slight angle.
    */

    Angle += DeltaVal;
    redraw( Dpy, buffers[render_buffer], Renderer, (Angle*M_PI)/180.0, 
	    GREENLINES, WIREFRAME);

    /*
    ** Check results 3. 
    */
    describe_test ("Frame 8: Red cube should be visible. Green wire cube in back.\n");
    
    testproc(Dpy, Win, 8);

    /*
    ** Swap buffers
    */
    XmbufDisplayBuffers (Dpy, 1, &buffers[render_buffer], 0, 0);
    XSync ( Dpy, False );  /* Push request over wire so we can check image. */
    displayed_buffer = render_buffer;
    render_buffer = !render_buffer;

    /*
    ** Check results 4. 
    */
    describe_test ("Frame 9: Buffers swapped. Green wire cube should be visible.\n");

    testproc(Dpy, Win, 9);

    /*
    ** Swap buffers
    */
    XmbufDisplayBuffers (Dpy, 1, &buffers[render_buffer], 0, 0);
    XSync ( Dpy, False );  /* Push request over wire so we can check image. */
    displayed_buffer = render_buffer;
    render_buffer = !render_buffer;

    /*
    ** Check results 5. 
    */
    describe_test ("Frame 10: Buffers swapped. Buffer should be blank.\n");

    testproc(Dpy, Win, 10);

    /*
     * Test is complete. Quit.
     */
    XCloseDisplay( Dpy );
    /*
     * Terminate the testing system.
     */
}


void testproc(Display *dpy, Window window, int cur_image)
{
    char image_name[80], test_name[80];

    sprintf(image_name,"mbx_centerline_%d", cur_image);
    sprintf(test_name, "MBX Centerline #%d", cur_image);
    image_testproc(image_name, test_name, img_desc,
		      dpy,window, 0, 0, 
		      glob_window_width, glob_window_height);
}


