/*
******************************************************************************
** PEXlib Texture Mapping Test Code 
**
** $Source: /src/rcs/Cats/XSystem/pex/txm_utility_code/convert_strings.c,v $
** $Revision: 510.1 $
** $Date: 94/02/26 10:42:16 $
** $Author: hmgr $
******************************************************************************
*/

/*
 *  THIS IS A PRELIMINARY VERSION!!
 */

/*
 * CONVERT_STRINGS.C -- Written to convert enumerated types from strings
 *                      to their corresponding values.
 *
 *                      Designed by Bradley L. Saunders     Nov. 16, 1993
 *                      Modified by Bradley L. Saunders     Dec. 20, 1993
 */


/*
 * This define points to the file that was created using 
 * the utility translate_strings 
 */

#define STRING_TABLE_FILE "./../txm_utility_code/string_table"

/******************************************************************************/
/*                                                                            */
/*  (c) Copyright Hewlett-Packard Company, 1993, Fort Collins, Colorado       */
/*                                                                            */
/*                            All Rights Reserved                             */
/*                                                                            */
/*  Permission to use, copy, modify, and distribute this software and its     */
/*  documentation for any purpose and without fee is hereby granted,          */
/*  provided that the above copyright notices appear in all copies and that   */
/*  both the copyright notices and this permission notice appear in           */
/*  supporting documentation, and that the name of Hewlett-Packard not be     */
/*  used in advertising or publicity pertaining to distribution of the        */
/*  software without specific, written prior permission.                      */
/*                                                                            */
/*  HEWLETT-PACKARD MAKES NO WARRANTY OF ANY KIND WITH REGARD TO THIS         */
/*  SOFTWARE, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/*  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  Hewlett-Packard    */
/*  shall not be liable for errors contained herein or direct, indirect,      */
/*  special, incidental or consequential damages in connection with the       */
/*  furnishing, performance or use of this software.                          */
/*                                                                            */
/******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <X11/PEX5/PEXlib.h>
#include "param_struct.h"
#include "prototypes.h"

char *string_table;

void init_string_table()
/*
 ******************************************************************************
 **    Description:
 **        This function loads the string table with all of the string values.
 **
 **    Author:
 **       Brad Saunders                Dec. 20, 1993
 **
 **    Input parameters:
 **        None.
 **
 **    Output parameters:
 **        None.
 **
 **    Entry Conditions:
 **        None.
 **
 **    Exit Conditions:
 **        None.
 **
 **    Error Conditions:
 **        None.
 **
 **    Referenced by:
 **        Parse_File().
 **
 **    Procedure/Function dependencies:
 **        None.
 **
 ******************************************************************************
 */
{
  unsigned long size;
  FILE *fp;
  char errstr[ERRORSTRINGSIZE]; 

  fp = fopen(STRING_TABLE_FILE,"r");
  if(fp == NULL)
  {
    sprintf(errstr,"Couldn't open file '%s'\n",STRING_TABLE_FILE);
    Error(errstr);
  }

 /*
  * Get size of string file
  */
  fseek(fp, 0L, SEEK_END);
  size = ftell(fp);
  fseek(fp, 0L, SEEK_SET);

 /*
  * Allocate enough memory for the string table and read it in
  */
  string_table = (char *)malloc(size);
  fread(string_table, size, 1, fp);
  fclose(fp);
}

PEXEnumTypeIndex Convert_Enum(char *what)
/*
 ******************************************************************************
 **    Description:
 **        This function converts a string to its respective Enumerated value.
 **        Returns a -1 if the string is not found and reports a warning.
 **
 **    Author:
 **       Brad Saunders                Dec. 20, 1993
 **
 **    Input parameters:
 **        char *what - Constant Enum Type string.
 **
 **    Output parameters:
 **        int - value associated with the string.
 **
 **    Entry Conditions:
 **        None.
 **
 **    Exit Conditions:
 **        None.
 **
 **    Error Conditions:
 **        None.
 **
 **    Referenced by:
 **        Nearly all functions in parser module.
 **
 **    Procedure/Function dependencies:
 **        init_string_table().
 **
 ******************************************************************************
 */
{
  char *marker;
  long value;
  char errstr[ERRORSTRINGSIZE];

  marker = strstr(string_table, what);
  if(marker == NULL) 
  {
    sprintf(errstr, "Enum not found - %s",what);
    Warning(errstr);
    return( (PEXEnumTypeIndex) -1 );
  }
  else 
  {
    sscanf(marker, "%*s %d", &value);
    return((PEXEnumTypeIndex) value);
  }
} 

int Convert_Constant(char *what)
/*
 ******************************************************************************
 **    Description:
 **        This function converts a string to its respective constant value.
 **        Returns a -1 if the string is not found and reports a warning.
 **
 **    Author:
 **       Brad Saunders                Dec. 20, 1993
 **
 **    Input parameters:
 **        char *what - Constant string.
 **
 **    Output parameters:
 **        int - value associated with the string.
 **
 **    Entry Conditions:
 **        None.
 **
 **    Exit Conditions:
 **        None.
 **
 **    Error Conditions:
 **        None.
 **
 **    Referenced by:
 **        Nearly all functions in parser module.
 **
 **    Procedure/Function dependencies:
 **        init_string_table().
 **
 ******************************************************************************
 */
{
  char *marker;
  int value;
  char errstr[ERRORSTRINGSIZE];

  marker = strstr(string_table, what);
  if(marker == NULL) 
  {
    sprintf(errstr, "Constant not found - %s",what);
    Warning(errstr);
    return(-1);
  }
  else 
  {
    sscanf(marker, "%*s %d", &value);
    return(value);
  }
} 

